#ifndef SCOUT_STRUCT_H
#define SCOUT_STRUCT_H


/* ------------------------------------------------------------------------
   -- COMMON STRUCTURES
   -- 
*/

typedef struct SCOUT_CONFIG {
    double x;
    double y;
    double theta;
} SCOUT_CONFIG;

typedef struct SCOUT_POS {
    double x;
    double y;
} SCOUT_POS;

typedef enum SCOUT_BOOL {
  SCOUT_TRUE=1,
  SCOUT_FALSE=0
} SCOUT_BOOL;

/* ------------------------------------------------------------------------
   -- ODOMETRY
   -- 
*/

typedef struct SCOUT_WHEELS_SPEED {
  double vr, vl;
} SCOUT_WHEELS_SPEED;

typedef struct SCOUT_SPEEDS {
  double v, w;
} SCOUT_SPEEDS;

typedef struct SCOUT_POS_STR {
  double x;				/* abcisse point commande' */
  double y;				/* ordonne'e point commande' */
  double xC;				/* abcisse centre robot */
  double yC;				/* ordonnee centre robot */
  double theta;		         	/* Angle robot */
  double v;				/* vitesse lineaire */
  double w;				/* vitesse angulaire */
} SCOUT_POS_STR;


/* Incertitudes sur x, y et theta */
typedef struct SCOUT_ODO_VAR_STR {
  double var[6];
} SCOUT_ODO_VAR_STR;

/* Structure complete de l'odo */
typedef struct SCOUT_ODO_STR {
  SCOUT_POS_STR robot;
  SCOUT_ODO_VAR_STR odoError;
  SCOUT_WHEELS_SPEED wheels;
} SCOUT_ODO_STR;

/* Nouvelle position du robot (centre) */
typedef struct SCOUT_SET_POS_STR {
  double x;
  double y;
  double theta;
} SCOUT_SET_POS_STR;

/* ------------------------------------------------------------------------
   -- CONNECTION
   -- 
*/
typedef enum SCOUT_CONNECT_MODE {
  SCOUT_NOT_CONNECTED,
  SCOUT_NDIRECT_CONNECTION,
  SCOUT_NCLIENT_CONNECTION,
  SCOUT_NCLIENTSIMUL_CONNECTION
} SCOUT_CONNECT_MODE;

typedef struct SCOUT_CONNECT_PARAM {
  SCOUT_CONNECT_MODE connectMode; 
  char host[16];			/* nom de machine */
  int  port;				/* numero de port */
  int  robotId;				/* numero de robot */
} SCOUT_CONNECT_PARAM;


/* ------------------------------------------------------------------------
   -- MOTION / REFERENCE
   -- 
*/

/*
 *  Arc of circle (or straight line)
 */
typedef struct SCOUT_TURN {
  double dl;  /* Longueur curviligne en metre
		     (>0: marche avant; <0: marche arriere) */
  double dth;   /* Variation angulaire en radian
		     (>0: trigo; <0: horloge) */
  int speedPercent;  /* A value in [0..100] */
  int unused;
} SCOUT_TURN;


typedef enum SCOUT_MOTION_TYPE {
  SCOUT_NO_MOTION,
  SCOUT_NOMADIC_MOTION,
  SCOUT_SERVO_POS,
  SCOUT_SERVO_CONFIG,
  SCOUT_SERVO_SPEED
} SCOUT_MOTION_TYPE;

#define SCOUT_MAX_POSTER_NAME 32

/* Poster tracking name */
typedef struct SCOUT_POSTER_NAME{
  char name[SCOUT_MAX_POSTER_NAME];
} SCOUT_POSTER_NAME;

/* Consigne pour le point commande' du robot
 *
 * Remarque : Pour des raison de non-holonomie et de statibilite de la 
 *            commande en theta, on ne controle pas le point sur l'axe des 
 *            roues mais un point a` une distance "distPoint" de cet axe
 *            (en avant ou en arriere de cet axe selon le sens de la marche 
 *            du robot).
 */
typedef enum SCOUT_MOTION_DIRECTION {
    SCOUT_FORWARD_MOTION,
    SCOUT_BACKWARD_MOTION
} SCOUT_MOTION_DIRECTION;

typedef struct SCOUT_REF_STR {
    SCOUT_MOTION_DIRECTION backFlag;	/* FORWARD ou BACKWARD */
    int numRef;
    double x;
    double y;
    double theta;
    double v;
    double w;
    double vmax;
    double wmax;
    double linAccelMax;
    double angAccelMax;
} SCOUT_REF_STR;

/* Parametre de l'asservissement en position (PI) */
typedef struct SCOUT_CMD_PARAM_STR {
    double distPoint;        /* Distance au point commande' */
    double KpLongit;         /* Coeff. proportionnel ecart longit dL */
    double KiLongit;         /* Coeff. integral ecart longitudinal */
    double KpTransv;         /* Coeff. proportionnel ecart transv. dT */
    double KiTransv;         /* Coeff. integral ecart transversal */
    double KpAng;            /* Coeff. proportionnel ecart angulaire */
    double KiAng;            /* Coeff. integral ecart angulaire */
    double longitErrorMax;   /* si dL > longitErrorMax, alors SdL = 0 */
    double transvErrorMax;   /* si dT > transvErrorMax, alors SdT = 0 */
    double angErrorMax;      /* si dT > transvErrorMax, alors SdA = 0 */
    double longitMaxErrSum;  /* Borne la somme des erreurs longit. SdL */
    double transvMaxErrSum;  /* Borne la somme des erreurs transv. SdT */
    double angMaxErrSum;     /* Borne la somme des erreurs transv. SdA */
} SCOUT_CMD_PARAM_STR;

typedef struct SCOUT_SPEED_REF_STR {
  int numRef;                /* Number of the reference */
  int updatePeriod;          /* Number of scout period to reach the reference */
  double v;                  /* Vitesse lineaire */
  double w;                  /* Vitesse angulaire */
  double vmax;
  double wmax;
  double linAccelMax;
  double angAccelMax;
} SCOUT_SPEED_REF_STR;

typedef enum SCOUT_TRACK_MODE {
  SCOUT_NO_TRACKING,
  SCOUT_TRACKING
} SCOUT_TRACK_MODE;

/* Informations concernant le tracking  */
typedef struct SCOUT_TRACK_STR {
  SCOUT_TRACK_MODE scoutCmdTrackMode;  
  int posterId;                  /* Id du poster tracke' */
  SCOUT_POSTER_NAME posterName;        /* Nom du poster tracke */
} SCOUT_TRACK_STR;


/* ------------------------------------------------------------------------
   -- BUMPERS
   -- 
*/
#define SCOUT_NB_BUMPERS  6

typedef struct SCOUT_BUMBERS_STATE {
  SCOUT_BOOL bumper[SCOUT_NB_BUMPERS]; 
} SCOUT_BUMBERS_STATE;

/* Bumpers position in robot frame (constant) */
typedef struct SCOUT_BUMPERS_POS {
    SCOUT_POS bumperPos[SCOUT_NB_BUMPERS];
} SCOUT_BUMPERS_POS;

/* ------------------------------------------------------------------------
   -- SONARS
   -- 
*/

#define SCOUT_NB_SONARS 16

/* Sonars position in robot frame (constant) */
typedef struct SCOUT_SONARS_CONFIG {
    SCOUT_CONFIG sonarConfig[SCOUT_NB_SONARS];
} SCOUT_SONARS_CONFIG;

/* Sonar raw echo */
typedef struct SCOUT_SONARS_ECHO_RHO {
  double rho[SCOUT_NB_SONARS];
} SCOUT_SONARS_ECHO_RHO;

/* Sonars echo in robot frame */
typedef struct SCOUT_SONARS_ECHO_XY {
    SCOUT_POS point[SCOUT_NB_SONARS];
} SCOUT_SONARS_ECHO_XY;

/* Sonars activation order */
typedef struct SCOUT_SONARS_FIRING_ORDER {
  int fireRate;                     /* Time between 2 sonars are fired.
				       1 period = 4 millisecond. */
  int unused;
  int firedSonars[SCOUT_NB_SONARS];   /* Firing order (255 to end the list) */
} SCOUT_SONARS_FIRING_ORDER;

/* ------------------------------------------------------------------------
   -- ALARMS
   -- 
*/
typedef struct SCOUT_ALARMS {
  int nbAlarms;
  int chocFlag;
  int lowBatteryFlag;     /* 0: OK; 1: medium; 2: low */
  int joystickInUseFlag;
  int estopDownFlag;
  int motionErrorFlag;
  int noMotionFlag;
  int importantDriftFlag;
  int refNotUpdatedFlag;
} SCOUT_ALARMS;

typedef enum SCOUT_ALARMS_TYPE {
  SCOUT_ALARM_CHOC,
  SCOUT_ALARM_BATTERY,
  SCOUT_ALARM_JOYSTICK,
  SCOUT_ALARM_ESTOP,
  SCOUT_ALARM_MOTION_ERROR,
  SCOUT_ALARM_NO_MOTION,
  SCOUT_ALARM_DRIFT,
  SCOUT_ALARM_REF_NOT_UPDATE
} SCOUT_ALARMS_TYPE;

typedef enum SCOUT_BATTERY_STATE {
  SCOUT_BATTERY_LOW,
  SCOUT_BATTERY_MEDIUM,
  SCOUT_BATTERY_HIGHT
} SCOUT_BATTERY_STATE;

typedef struct SCOUT_BATTERIES_STR {
  SCOUT_BATTERY_STATE charge;
  int unused;
  double levelCpu;      /* MIN: 10.8V */
  double levelMotor;    /* MIN: 10.8V */
} SCOUT_BATTERIES_STR;

#endif /* SCOUT_STRUCT_H */
